package javaCodeGen;

import CodeGen.IDataType;
import MDL.Param;



public abstract class JavaDataType extends JavaElement implements IDataType{
	
 	/**
 	 * Factory
 	 */
	public static JavaDataType getDataType(Param.Type type){
		JavaDataType result;
		if(type.equals(Param.Type.Boolean)){
			result= new JavaBoolean();
		}else if(type.equals(Param.Type.UnsignedInt)){
			result= new JavaUnsignedInt(); 
		}else if(type.equals(Param.Type.Integer)){
			result= new JavaInt();
		}else if(type.equals(Param.Type.Chars)){
			result= new JavaChars();
		}else if(type.equals(Param.Type.Bytes)){
			result = new JavaBytes();
		}else{
			return null;
		}		
		return result;		
	}
	
	/**
	 * Given the type and byte length, what data type should
	 * implement this parameter?
	 * @param length byte length of parameter
	 * @return string representation of data type
	 */
	public abstract String getType(int length);
	
	/**
	 * Given a value and a number of bytes, what should the byte
	 * array look like?
	 * @param value value to be represented
	 * @param length number of bytes
	 * @return string representation of a byte array
	 */
	public abstract String[] getByteArrayRep(String value, int length);

	/**
	 * Generate a method, that given a byte array, return its value. 
	 * The return value of the method should of whatever type this is
	 * @param varname the name of the byte array that will be converted
	 */
	public abstract void genByteArrayConverstion(String varname);
	
}
