package modules;

/*
 *YellowJacket Servo uses WASP protocol
 */
public class Servo{
     /**
      * this could be useful when working in the next layer in the packet
      * @return the module's name
      */
     public String getModuleName(){
          return "Servo";
     }
     
     /*
      * The Follow Set of Constants are for the putServo method.
      */
     //Represents reach component of the packet
     private static byte[][] putServo_components = {
          {2, }, //versionNum: 
          new byte[6], //MAC: 
          {32, }, //methNum: 
          {0, }, //respFlag: Value is 0 if sending.
          {6, }, //dataLen: 
          {'s', 'r', 'v', 'o', }, //methName: 
          new byte[1], //servoNum: 
          new byte[1], //pos: Servo position.  A value between 1 and 255
          {50, }, //checkSum: 
     };
     //indicates which of the values are constant
     //and which ones will be determined at run time
     private static boolean[] putServo_const = {
          true, false, true, true, true, true, false, false, true, 
     };
     //the total length of the packet
     //note that this could be calculated at run time
     private static int putServo_len = 17;
     
     /**
      *creates a packet
      *moves the servo to a certain position
      *@param MAC 
      *@param servoNum 
      *@param pos Servo position.  A value between 1 and 255
      *@return array of bytes representing the data packet
      */
     public static byte[] putServo(byte[] MAC,int servoNum,int pos) throws Exception{
          int i_MAC = 1;
          putServo_components[i_MAC]= MAC;
          int i_servoNum = 6;
          putServo_components[i_servoNum][0] = (byte) servoNum;
          int i_pos = 7;
          putServo_components[i_pos][0] = (byte) pos;
          byte[] result = Utils.merge(putServo_components, putServo_len);
          return result;
     }
          
     /*
      * If all the constant values for a putServo match then, its this type of packet
      * If all the constant values for a putServo match then, its this type of packet
      * @param packet
      * @return true if all constant values match
      */
     public static boolean isputServoPacket(byte[] packet){
          //If these are not the same length, then they are not the same packet.
          boolean matches = (packet.length ==putServo_len);
          int bytesSoFar = 0;
          for(int component = 0; component < putServo_components.length && matches; component++){
               for(int bytesIn = 0; bytesIn < putServo_components[component][bytesIn]; bytesIn++){
                    if(putServo_const[component]){ //If it is a constant value...
                         //make sure that it matches.
                         matches = packet[bytesSoFar]==putServo_components[component][bytesIn];
                    }
                    bytesSoFar++;
               }
          }
          return matches;
     }
     /**
      *gets the value of putServo
      *@param dataPacket : the packet to parse
      *should be formatted like a putServopacket
      *@Exception if it itnot the proper type of packet
      *@return putServo:
      */
     public static byte[] getputServo_MAC(byte[] datapacket)throws Exception{
     if(!Servo.isputServoPacket(datapacket)){ throw new Exception();}
          int depthIntoPacket = 1;
          byte[] component = new byte[6];
          for(int i=0;i < 6; i++){
               component[i] = datapacket[i+ depthIntoPacket];
          }
          return component;
     }
     /**
      *gets the value of putServo
      *@param dataPacket : the packet to parse
      *should be formatted like a putServopacket
      *@Exception if it itnot the proper type of packet
      *@return putServo:
      */
     public static int getputServo_servoNum(byte[] datapacket)throws Exception{
     if(!Servo.isputServoPacket(datapacket)){ throw new Exception();}
          int depthIntoPacket = 14;
          byte[] component = new byte[1];
          for(int i=0;i < 1; i++){
               component[i] = datapacket[i+ depthIntoPacket];
          }
          int value =0;
          for(int i = 0; i < component.length; i++){
               int dig= component[i];
               value = (int) (value + dig*(Math.pow(2, i)));
          }
          return value;
     }
     /**
      *gets the value of putServo
      *@param dataPacket : the packet to parse
      *should be formatted like a putServopacket
      *@Exception if it itnot the proper type of packet
      *@return putServo:Servo position.  A value between 1 and 255
      */
     public static int getputServo_pos(byte[] datapacket)throws Exception{
     if(!Servo.isputServoPacket(datapacket)){ throw new Exception();}
          int depthIntoPacket = 15;
          byte[] component = new byte[1];
          for(int i=0;i < 1; i++){
               component[i] = datapacket[i+ depthIntoPacket];
          }
          int value =0;
          for(int i = 0; i < component.length; i++){
               int dig= component[i];
               value = (int) (value + dig*(Math.pow(2, i)));
          }
          return value;
     }
}

